__title__ = '2ip'
__description__ = '2ip.me API Client.'
__url__ = 'https://github.com/python-modules/2ip'
__version__ = '0.0.1'
__author__ = 'gbe0'
__author_email__ = 'python@gbe0.com'
__license__ = 'MIT'
__copyright__ = 'Copyright 2021 gbe0'